@@ -1,8 +1,8 @@
 /* result.c - wait for an ldap result */
-/* $OpenLDAP: pkg/ldap/libraries/libldap/result.c,v 1.84.2.3 2003/12/01 00:58:26 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/result.c,v 1.99.2.2 2005/05/06 16:10:22 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2003 The OpenLDAP Foundation.
+ * Copyright 1998-2005 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -73,7 +73,7 @@ static int ldap_mark_abandoned LDAP_P(( LDAP *ld, ber_int_t msgid ));
 static int wait4msg LDAP_P(( LDAP *ld, ber_int_t msgid, int all, struct timeval *timeout,
 	LDAPMessage **result ));
 static ber_tag_t try_read1msg LDAP_P(( LDAP *ld, ber_int_t msgid,
-	int all, Sockbuf *sb, LDAPConn *lc, LDAPMessage **result ));
+	int all, Sockbuf *sb, LDAPConn **lc, LDAPMessage **result ));
 static ber_tag_t build_result_ber LDAP_P(( LDAP *ld, BerElement **bp, LDAPRequest *lr ));
 static void merge_error_info LDAP_P(( LDAP *ld, LDAPRequest *parentr, LDAPRequest *lr ));
 static LDAPMessage * chkResponseList LDAP_P(( LDAP *ld, int msgid, int all));
@@ -111,11 +111,7 @@ ldap_result(
 	assert( ld != NULL );
 	assert( result != NULL );
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( OPERATION, ARGS, "ldap_result msgid %d\n", msgid, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_result msgid %d\n", msgid, 0, 0 );
-#endif
 
 #ifdef LDAP_R_COMPILE
 	ldap_pvt_thread_mutex_lock( &ld->ld_res_mutex );
@@ -149,27 +145,17 @@ chkResponseList(
 	 * wait until it arrives or timeout occurs.
 	 */
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( OPERATION, ARGS, "ldap_chkResponseList for msgid=%d, all=%d\n", 
-		msgid, all, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE,
 		"ldap_chkResponseList for msgid=%d, all=%d\n",
 	    msgid, all, 0 );
-#endif
 	lastlm = NULL;
 	for ( lm = ld->ld_responses; lm != NULL; lm = nextlm ) {
 		nextlm = lm->lm_next;
 
 		if ( ldap_abandoned( ld, lm->lm_msgid ) ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( OPERATION, DETAIL1, 
-				"ldap_chkResponseList msg abandoned, msgid %d\n", msgid, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE,
 				"ldap_chkResponseList msg abandoned, msgid %d\n",
 			    msgid, 0, 0 );
-#endif
 			ldap_mark_abandoned( ld, lm->lm_msgid );
 
 			if ( lastlm == NULL ) {
@@ -191,12 +177,27 @@ chkResponseList(
 				break;
 			}
 
-			for ( tmp = lm; tmp != NULL; tmp = tmp->lm_chain ) {
-				if ( tmp->lm_msgtype != LDAP_RES_SEARCH_ENTRY
-				    && tmp->lm_msgtype != LDAP_RES_SEARCH_REFERENCE
-					&& tmp->lm_msgtype != LDAP_RES_INTERMEDIATE )
-				{
-					break;
+			if ( lm->lm_chain == NULL ) {
+				assert(lm->lm_chain_tail == lm);
+				if ((lm->lm_msgtype == LDAP_RES_SEARCH_ENTRY) ||
+					(lm->lm_msgtype == LDAP_RES_SEARCH_REFERENCE) ||
+					(lm->lm_msgtype == LDAP_RES_INTERMEDIATE)) {
+					tmp = NULL;
+				} else {
+					tmp = lm;
+				}
+			} else {
+				assert(lm->lm_chain_tail);
+				assert(lm->lm_chain_tail->lm_chain);
+				if ((lm->lm_chain_tail->lm_chain->lm_msgtype
+						== LDAP_RES_SEARCH_ENTRY) ||
+					(lm->lm_chain_tail->lm_chain->lm_msgtype
+						== LDAP_RES_SEARCH_REFERENCE) ||
+					(lm->lm_chain_tail->lm_chain->lm_msgtype
+						== LDAP_RES_INTERMEDIATE)) {
+					tmp = NULL;
+				} else {
+					tmp = lm->lm_chain_tail->lm_chain;
 				}
 			}
 
@@ -219,31 +220,23 @@ chkResponseList(
 		        ? lm->lm_chain : lm->lm_next);
 	    }
 	    if ( all == LDAP_MSG_ONE && lm->lm_chain != NULL ) {
-		    lm->lm_chain->lm_next = lm->lm_next;
-		    lm->lm_chain = NULL;
+			lm->lm_chain->lm_next = lm->lm_next;
+			lm->lm_chain->lm_chain_tail = ( lm->lm_chain_tail != lm ) ? lm->lm_chain_tail : lm->lm_chain;
+			assert(lm->lm_chain->lm_chain_tail);
+			lm->lm_chain = NULL;
+			lm->lm_chain_tail = NULL;
 	    }
 	    lm->lm_next = NULL;
     }
 
 #ifdef LDAP_DEBUG
 	if( lm == NULL) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, RESULTS, "ldap_chkResponseList returns NULL\n",
-			0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE,
 			"ldap_chkResponseList returns NULL\n", 0, 0, 0);
-#endif
 	} else {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, RESULTS, 
-			"ldap_chkResponseList returns msgid %d, type 0x%02lu\n",
-			lm->lm_msgid, (unsigned long) lm->lm_msgtype, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE,
 			"ldap_chkResponseList returns msgid %d, type 0x%02lu\n",
 			lm->lm_msgid, (unsigned long) lm->lm_msgtype, 0);
-#endif
 	}
 #endif
     return lm;
@@ -268,22 +261,11 @@ wait4msg(
 
 #ifdef LDAP_DEBUG
 	if ( timeout == NULL ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, ARGS, 
-			"wait4msg (infinite timeout), msgid %d\n", msgid, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE, "wait4msg (infinite timeout), msgid %d\n",
 		    msgid, 0, 0 );
-#endif
 	} else {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, ARGS, 
-			"wait4msg (timeout %ld sec, %ld usec), msgid %d\n", 
-			(long) timeout->tv_sec, (long) timeout->tv_usec, msgid );
-#else
 		Debug( LDAP_DEBUG_TRACE, "wait4msg (timeout %ld sec, %ld usec), msgid %d\n",
 		       (long) timeout->tv_sec, (long) timeout->tv_usec, msgid );
-#endif
 	}
 #endif /* LDAP_DEBUG */
 
@@ -298,13 +280,8 @@ wait4msg(
 	rc = -2;
 	while ( rc == -2 ) {
 #ifdef LDAP_DEBUG
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, ARGS, 
-			"wait4msg continue, msgid %d, all %d\n", msgid, all, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE, "wait4msg continue, msgid %d, all %d\n",
 		    msgid, all, 0 );
-#endif
 		if ( ldap_debug & LDAP_DEBUG_TRACE ) {
 			ldap_dump_connection( ld, ld->ld_conns, 1 );
 			ldap_dump_requests_and_responses( ld );
@@ -314,29 +291,26 @@ wait4msg(
         if( (*result = chkResponseList(ld, msgid, all)) != NULL ) {
             rc = (*result)->lm_msgtype;
         } else {
+			int lc_ready = 0;
 
-			for ( lc = ld->ld_conns; lc != NULL; lc = lc->lconn_next ) {
+			for ( lc = ld->ld_conns; lc != NULL; lc = nextlc ) {
+				nextlc = lc->lconn_next;
 				if ( ber_sockbuf_ctrl( lc->lconn_sb,
 						LBER_SB_OPT_DATA_READY, NULL ) ) {
-					    rc = try_read1msg( ld, msgid, all, lc->lconn_sb,
-					        lc, result );
+					rc = try_read1msg( ld, msgid, all, lc->lconn_sb,
+						&lc, result );
+					lc_ready = 1;
 				    break;
 				}
 	        }
 
-		    if ( lc == NULL ) {
+		    if ( !lc_ready ) {
 			    rc = ldap_int_select( ld, tvp );
 #ifdef LDAP_DEBUG
 			    if ( rc == -1 ) {
-#ifdef NEW_LOGGING
-					LDAP_LOG ( OPERATION, ARGS, 
-						"wait4msg: ldap_int_select returned -1: errno %d\n", 
-						errno, 0, 0 );
-#else
 			        Debug( LDAP_DEBUG_TRACE,
 				        "ldap_int_select returned -1: errno %d\n",
 				        errno, 0, 0 );
-#endif
 			    }
 #endif
 
@@ -373,7 +347,8 @@ wait4msg(
 					        ldap_is_read_ready( ld,
 					        lc->lconn_sb )) {
 						    rc = try_read1msg( ld, msgid, all,
-						        lc->lconn_sb, lc, result );
+						        lc->lconn_sb, &lc, result );
+							if ( lc == NULL ) lc = nextlc;
 					    }
 				    }
 			    }
@@ -388,13 +363,8 @@ wait4msg(
 				break;
 			}
 
-#ifdef NEW_LOGGING
-			LDAP_LOG ( OPERATION, DETAIL1, 
-				"wait4msg: %ld secs to go\n", (long) tv.tv_sec, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE, "wait4msg:  %ld secs to go\n",
 			       (long) tv.tv_sec, 0, 0 );
-#endif
 			start_time = tmp_time;
 		}
 	}
@@ -409,16 +379,17 @@ try_read1msg(
 	ber_int_t msgid,
 	int all,
 	Sockbuf *sb,
-	LDAPConn *lc,
+	LDAPConn **lcp,
 	LDAPMessage **result )
 {
 	BerElement	*ber;
-	LDAPMessage	*new, *l, *prev, *tmp;
+	LDAPMessage	*new, *l, *prev, *tmp, *chain_head;
 	ber_int_t	id;
 	ber_tag_t	tag;
 	ber_len_t	len;
 	int		foundit = 0;
 	LDAPRequest	*lr, *tmplr;
+	LDAPConn	*lc;
 	BerElement	tmpber;
 	int		rc, refer_cnt, hadref, simple_request;
 	ber_int_t	lderr;
@@ -432,13 +403,12 @@ try_read1msg(
 	int     v3ref;
 
 	assert( ld != NULL );
-	assert( lc != NULL );
+	assert( lcp != NULL );
+	assert( *lcp != NULL );
 	
-#ifdef NEW_LOGGING
-	LDAP_LOG ( OPERATION, ARGS, "read1msg: msgid %d, all %d\n", msgid, all, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "read1msg: msgid %d, all %d\n", msgid, all, 0 );
-#endif
+
+	lc = *lcp;
 
 retry:
 	if ( lc->lconn_ber == NULL ) {
@@ -473,14 +443,9 @@ nextresp3:
 	if ( tag != LDAP_TAG_MESSAGE ) {
 		if ( tag == LBER_DEFAULT) {
 #ifdef LDAP_DEBUG		   
-#ifdef NEW_LOGGING
-			LDAP_LOG ( OPERATION, DETAIL1, 
-				"read1msg: ber_get_next failed\n", 0, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_CONNS,
 			      "ber_get_next failed.\n", 0, 0, 0 );
 #endif		   
-#endif		   
 #ifdef EWOULDBLOCK			
 			if (errno==EWOULDBLOCK) return -2;
 #endif
@@ -503,11 +468,7 @@ nextresp3:
 
 	/* if it's been abandoned, toss it */
 	if ( ldap_abandoned( ld, id ) ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, DETAIL1, "read1msg: abandoned\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "abandoned\n", 0, 0, 0);
-#endif
 retry_ber:
 		ber_free( ber, 1 );
 		if ( ber_sockbuf_ctrl( sb, LBER_SB_OPT_DATA_READY, NULL ) ) {
@@ -517,15 +478,9 @@ retry_ber:
 	}
 
 	if (( lr = ldap_find_request_by_msgid( ld, id )) == NULL ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, DETAIL1, 
-			"read1msg: no request for response with msgid %ld (tossing)\n",
-			(long) id, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 		    "no request for response with msgid %ld (tossing)\n",
 		    (long) id, 0, 0 );
-#endif
 		goto retry_ber;
 	}
 #ifdef LDAP_CONNECTIONLESS
@@ -541,17 +496,10 @@ nextresp2:
 		return( -1 );
 	}
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( OPERATION, DETAIL1, 
-		"read1msg: ldap_read: message type %s msgid %ld, original id %ld\n",
-	    ldap_int_msgtype2str( tag ),
-		(long) lr->lr_msgid, (long) lr->lr_origid );
-#else
 	Debug( LDAP_DEBUG_TRACE,
 		"ldap_read: message type %s msgid %ld, original id %ld\n",
 	    ldap_int_msgtype2str( tag ),
 		(long) lr->lr_msgid, (long) lr->lr_origid );
-#endif
 
 	id = lr->lr_origid;
 	refer_cnt = 0;
@@ -586,16 +534,9 @@ nextresp2:
 						/* If haven't got end search, set chasing referrals */
 						if( lr->lr_status != LDAP_REQST_COMPLETED) {
 							lr->lr_status = LDAP_REQST_CHASINGREFS;
-#ifdef NEW_LOGGING
-							LDAP_LOG ( OPERATION, DETAIL1, 
-								"read1msg: search ref chased,"
-								"mark request chasing refs, id =	%d\n",
-								lr->lr_msgid, 0, 0 );
-#else
 							Debug( LDAP_DEBUG_TRACE,
 							    "read1msg:  search ref chased, mark request chasing refs, id = %d\n",
 							    lr->lr_msgid, 0, 0);
-#endif
 						}
 						v3ref = 1;	/* We sucessfully chased the reference */
 					}
@@ -619,16 +560,9 @@ nextresp2:
 						if( ber_scanf( &tmpber, "{v}", &refs) == LBER_ERROR) {
 							rc = LDAP_DECODING_ERROR;
 							lr->lr_status = LDAP_REQST_COMPLETED;
-#ifdef NEW_LOGGING
-							LDAP_LOG ( OPERATION, DETAIL1, 
-								"read1msg: referral decode error,"
-								"mark request completed, id =	%d\n",
-								lr->lr_msgid, 0, 0 );
-#else
 							Debug( LDAP_DEBUG_TRACE,
 							    "read1msg: referral decode error, mark request completed, id = %d\n",
 								    lr->lr_msgid, 0, 0);
-#endif
 						} else {
 							/* Chase the referral 
 							 * Note: refs arrary is freed by ldap_chase_v3referrals
@@ -636,16 +570,9 @@ nextresp2:
 							refer_cnt = ldap_chase_v3referrals( ld, lr, refs,
 							    0, &lr->lr_res_error, &hadref );
 							lr->lr_status = LDAP_REQST_COMPLETED;
-#ifdef NEW_LOGGING
-							LDAP_LOG ( OPERATION, DETAIL1, 
-								"read1msg: referral chased,"
-								"mark request completed, id =	%d\n",
-								lr->lr_msgid, 0, 0 );
-#else
 							Debug( LDAP_DEBUG_TRACE,
 							    "read1msg:  referral chased, mark request completed, id = %d\n",
 							    lr->lr_msgid, 0, 0);
-#endif
 							if( refer_cnt > 0) {
 								v3ref = 1;  /* Referral successfully chased */
 							}
@@ -694,15 +621,8 @@ nextresp2:
 					refer_cnt = ldap_chase_referrals( ld, lr,
 						&lr->lr_res_error, -1, &hadref );
 					lr->lr_status = LDAP_REQST_COMPLETED;
-#ifdef NEW_LOGGING
-					LDAP_LOG ( OPERATION, DETAIL1, 
-						"read1msg: V2 referral chased,"
-						"mark request completed, id =	%d\n",
-						lr->lr_msgid, 0, 0 );
-#else
 					Debug( LDAP_DEBUG_TRACE,
 					    "read1msg:  V2 referral chased, mark request completed, id = %d\n", lr->lr_msgid, 0, 0);
-#endif
 				}
 
 				/* save errno, message, and matched string */
@@ -715,27 +635,15 @@ nextresp2:
 				} else {
 					lr->lr_res_errno = LDAP_PARTIAL_RESULTS;
 				}
-#ifdef NEW_LOGGING
-LDAP_LOG ( OPERATION, DETAIL1, 
-	"read1msg: new result: res_errno: %d, res_error: <%s>, res_matched: <%s>\n",
-    lr->lr_res_errno, lr->lr_res_error ? lr->lr_res_error : "",
-    lr->lr_res_matched ? lr->lr_res_matched : "" );
-#else
 Debug( LDAP_DEBUG_TRACE,
     "new result:  res_errno: %d, res_error: <%s>, res_matched: <%s>\n",
     lr->lr_res_errno, lr->lr_res_error ? lr->lr_res_error : "",
     lr->lr_res_matched ? lr->lr_res_matched : "" );
-#endif
 			}
 		}
 
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, DETAIL1, "read1msg: %d new referrals\n", 
-			refer_cnt, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE,
 		    "read1msg:  %d new referrals\n", refer_cnt, 0, 0 );
-#endif
 
 		if ( refer_cnt != 0 ) {	/* chasing referrals */
 			ber_free( ber, 1 );
@@ -755,14 +663,8 @@ Debug( LDAP_DEBUG_TRACE,
 			}
 
 			lr->lr_status = LDAP_REQST_COMPLETED; /* declare this request done */
-#ifdef NEW_LOGGING
-			LDAP_LOG ( OPERATION, DETAIL1, 
-				"read1msg: mark request completed, id = %d\n", 
-				lr->lr_msgid, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE,
 			    "read1msg:  mark request completed, id = %d\n", lr->lr_msgid, 0, 0);
-#endif
 			while ( lr->lr_parent != NULL ) {
 				merge_error_info( ld, lr->lr_parent, lr );
 
@@ -775,32 +677,26 @@ Debug( LDAP_DEBUG_TRACE,
 			/* Check if all requests are finished, lr is now parent */
 			tmplr = lr;
 			if (tmplr->lr_status == LDAP_REQST_COMPLETED) {
-				for(tmplr=lr->lr_child; tmplr != NULL; tmplr=tmplr->lr_refnext) {
-				if( tmplr->lr_status != LDAP_REQST_COMPLETED) {
-					break;
-					}
+				for ( tmplr=lr->lr_child;
+					tmplr != NULL;
+					tmplr=tmplr->lr_refnext)
+				{
+					if( tmplr->lr_status != LDAP_REQST_COMPLETED) break;
 				}
 			}
 
 			/* This is the parent request if the request has referrals */
-			if ( lr->lr_outrefcnt <= 0 && lr->lr_parent == NULL && tmplr == NULL ) {
+			if ( lr->lr_outrefcnt <= 0 && lr->lr_parent == NULL &&
+				tmplr == NULL )
+			{
 				id = lr->lr_msgid;
 				tag = lr->lr_res_msgtype;
-#ifdef NEW_LOGGING
-			LDAP_LOG ( OPERATION, DETAIL1, 
-				"read1msg: request %ld done\n", (long) id, 0, 0 );
-			LDAP_LOG ( OPERATION, DETAIL1, 
-				"read1msg: res_errno: %d,res_error: <%s>, res_matched: <%s>\n",
-				lr->lr_res_errno, lr->lr_res_error ? lr->lr_res_error : "",
-				lr->lr_res_matched ? lr->lr_res_matched : "" );
-#else
 				Debug( LDAP_DEBUG_ANY, "request %ld done\n",
 				    (long) id, 0, 0 );
 Debug( LDAP_DEBUG_TRACE,
 "res_errno: %d, res_error: <%s>, res_matched: <%s>\n",
 lr->lr_res_errno, lr->lr_res_error ? lr->lr_res_error : "",
 lr->lr_res_matched ? lr->lr_res_matched : "" );
-#endif
 				if ( !simple_request ) {
 					ber_free( ber, 1 );
 					ber = NULL;
@@ -815,6 +711,7 @@ lr->lr_res_matched ? lr->lr_res_matched : "" );
 
 			if ( lc != NULL ) {
 				ldap_free_connection( ld, lc, 0, 1 );
+				*lcp = NULL;
 			}
 		}
 	}
@@ -882,8 +779,11 @@ lr->lr_res_matched ? lr->lr_res_matched : "" );
 				firstmsg = 0;
 				new->lm_next = ld->ld_responses;
 				ld->ld_responses = new;
+				new->lm_chain_tail = new;
+				chain_head = new;
 			} else {
 				tmp->lm_chain = new;
+				chain_head->lm_chain_tail = tmp;
 			}
 			tmp = new;
 			/* "ok" means there's more to parse */
@@ -905,6 +805,7 @@ lr->lr_res_matched ? lr->lr_res_matched : "" );
 		 * first response off the head of the chain.
 		 */
 			tmp->lm_chain = new;
+			chain_head->lm_chain_tail = tmp;
 			*result = chkResponseList( ld, msgid, all );
 			ld->ld_errno = LDAP_SUCCESS;
 			return( (*result)->lm_msgtype );
@@ -948,26 +849,33 @@ lr->lr_res_matched ? lr->lr_res_matched : "" );
 
 		new->lm_next = ld->ld_responses;
 		ld->ld_responses = new;
+		new->lm_chain_tail = new;
 		goto exit;
 	}
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( OPERATION, DETAIL1, 
-		"read1msg: adding response id %ld type %ld\n",
-		(long) new->lm_msgid, (long) new->lm_msgtype, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "adding response id %ld type %ld:\n",
 	    (long) new->lm_msgid, (long) new->lm_msgtype, 0 );
-#endif
 
 	/* part of a search response - add to end of list of entries */
-	for ( tmp = l; (tmp->lm_chain != NULL) &&
-	    	((tmp->lm_chain->lm_msgtype == LDAP_RES_SEARCH_ENTRY) ||
-	    	 (tmp->lm_chain->lm_msgtype == LDAP_RES_SEARCH_REFERENCE) ||
-			 (tmp->lm_chain->lm_msgtype == LDAP_RES_INTERMEDIATE ));
-	    tmp = tmp->lm_chain )
-		;	/* NULL */
-	tmp->lm_chain = new;
+	if (l->lm_chain == NULL) {
+		assert(l->lm_chain_tail == l);
+		l->lm_chain = new;
+	} else {
+		assert(l->lm_chain_tail);
+		assert(l->lm_chain_tail->lm_chain);
+		if ((l->lm_chain_tail->lm_chain->lm_msgtype
+				== LDAP_RES_SEARCH_ENTRY) ||
+			(l->lm_chain_tail->lm_chain->lm_msgtype
+				== LDAP_RES_SEARCH_REFERENCE) ||
+			(l->lm_chain_tail->lm_chain->lm_msgtype
+				== LDAP_RES_INTERMEDIATE)) {
+			l->lm_chain_tail->lm_chain->lm_chain = new;
+			l->lm_chain_tail = l->lm_chain_tail->lm_chain;
+		} else {
+			/*FIXME: ldap_msgfree( l->lm_chain_tail->lm_chain );*/
+			l->lm_chain_tail->lm_chain = new;
+		}
+	}
 
 	/* return the whole chain if that's what we were looking for */
 	if ( foundit ) {
@@ -1072,21 +980,12 @@ merge_error_info( LDAP *ld, LDAPRequest *parentr, LDAPRequest *lr )
 		}
 	}
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( OPERATION, DETAIL1, "merged parent (id %d) error info:  ",
-	    parentr->lr_msgid, 0, 0 );
-	LDAP_LOG( OPERATION, DETAIL1, "result errno %d, error <%s>, matched <%s>\n",
-	    parentr->lr_res_errno, parentr->lr_res_error ?
-	    parentr->lr_res_error : "", parentr->lr_res_matched ?
-	    parentr->lr_res_matched : "" );
-#else
 	Debug( LDAP_DEBUG_TRACE, "merged parent (id %d) error info:  ",
 	    parentr->lr_msgid, 0, 0 );
 	Debug( LDAP_DEBUG_TRACE, "result errno %d, error <%s>, matched <%s>\n",
 	    parentr->lr_res_errno, parentr->lr_res_error ?
 	    parentr->lr_res_error : "", parentr->lr_res_matched ?
 	    parentr->lr_res_matched : "" );
-#endif
 }
 
 
@@ -1132,11 +1031,7 @@ ldap_msgfree( LDAPMessage *lm )
 	LDAPMessage	*next;
 	int		type = 0;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( OPERATION, ENTRY, "ldap_msgfree\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_msgfree\n", 0, 0, 0 );
-#endif
 
 	for ( ; lm != NULL; lm = next ) {
 		next = lm->lm_chain;
@@ -1161,11 +1056,7 @@ ldap_msgdelete( LDAP *ld, int msgid )
 
 	assert( ld != NULL );
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( OPERATION, ENTRY, "ldap_msgdelete\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_msgdelete\n", 0, 0, 0 );
-#endif
 
 	prev = NULL;
 #ifdef LDAP_R_COMPILE
