@@ -1,5 +1,5 @@
 /* dn2id.c - routines to deal with the dn2id index */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/dn2id.c,v 1.106.2.3 2005/03/14 22:24:59 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/dn2id.c,v 1.106.2.4 2005/04/29 21:29:06 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 2000-2005 The OpenLDAP Foundation.
@@ -361,6 +361,7 @@ bdb_dn2idl(
 	((char *)key.data)[0] = prefix;
 	AC_MEMCPY( &((char *)key.data)[1], e->e_nname.bv_val, key.size - 1 );
 
+	BDB_IDL_ZERO( ids );
 	rc = bdb_idl_fetch_key( op->o_bd, db, NULL, &key, ids, NULL, 0 );
 
 	if( rc != 0 ) {
@@ -417,6 +418,9 @@ int hdb_fix_dn(
 	char *ptr, *nptr;
 	int max = 0;
 
+	if ( !e->e_id )
+		return 0;
+
 	/* count length of all DN components */
 	for ( ei = BEI(e); ei && ei->bei_id; ei=ei->bei_parent ) {
 		rlen += ei->bei_rdn.bv_len + 1;
@@ -497,11 +501,17 @@ hdb_dn2id_add(
 
 	DBTzero(&key);
 	DBTzero(&data);
-	key.data = &nid;
 	key.size = sizeof(ID);
 	key.flags = DB_DBT_USERMEM;
 	BDB_ID2DISK( eip->bei_id, &nid );
 
+	/* Delete parent's IDL cache entry */
+	if ( bdb->bi_idl_cache_size ) {
+		key.data = &eip->bei_id;
+		bdb_idl_cache_del( bdb, db, &key );
+	}
+	key.data = &nid;
+
 	/* Need to make dummy root node once. Subsequent attempts
 	 * will fail harmlessly.
 	 */
@@ -514,9 +524,6 @@ hdb_dn2id_add(
 		db->put( db, txn, &key, &data, DB_NODUPDATA );
 	}
 
-	if ( bdb->bi_idl_cache_size ) {
-		bdb_idl_cache_del( bdb, db, &key );
-	}
 	data.data = d;
 	data.size = sizeof(diskNode) + rlen + nrlen;
 	data.flags = DB_DBT_USERMEM;
@@ -554,7 +561,6 @@ hdb_dn2id_delete(
 	DBTzero(&key);
 	key.size = sizeof(ID);
 	key.ulen = key.size;
-	key.data = &nid;
 	key.flags = DB_DBT_USERMEM;
 	BDB_ID2DISK( eip->bei_id, &nid );
 
@@ -564,9 +570,16 @@ hdb_dn2id_delete(
 	data.dlen = data.size;
 	data.flags = DB_DBT_USERMEM | DB_DBT_PARTIAL;
 
+	/* Delete IDL cache entries */
 	if ( bdb->bi_idl_cache_size ) {
+		/* Ours */
+		key.data = &e->e_id;
+		bdb_idl_cache_del( bdb, db, &key );
+		/* Parent's */
+		key.data = &eip->bei_id;
 		bdb_idl_cache_del( bdb, db, &key );
 	}
+	key.data = &nid;
 	rc = db->cursor( db, txn, &cursor, bdb->bi_db_opflags );
 	if ( rc ) return rc;
 
@@ -800,7 +813,7 @@ struct dn2id_cookie {
 	ID dbuf;
 	ID *ids;
 	void *ptr;
-	ID tmp[BDB_IDL_DB_SIZE];
+	ID tmp[BDB_IDL_UM_SIZE];
 	ID *buf;
 	DBT key;
 	DBT data;
@@ -816,7 +829,7 @@ apply_func(
 	EntryInfo *ei = data;
 	ID *idl = arg;
 
-	bdb_idl_insert( idl, ei->bei_id );
+	bdb_idl_append_one( idl, ei->bei_id );
 	return 0;
 }
 
@@ -911,7 +924,7 @@ hdb_dn2idl_internal(
 					ei.bei_rdn.bv_len = len - sizeof(diskNode)
 						- ei.bei_nrdn.bv_len;
 					ei.bei_rdn.bv_val = d->nrdn + ei.bei_nrdn.bv_len + 1;
-					bdb_idl_insert( cx->tmp, ei.bei_id );
+					bdb_idl_append_one( cx->tmp, ei.bei_id );
 					hdb_cache_load( cx->bdb, &ei, &ei2 );
 				}
 			}
@@ -941,16 +954,14 @@ saveit:
 gotit:
 	if ( !BDB_IDL_IS_ZERO( cx->tmp )) {
 		if ( cx->prefix == DN_SUBTREE_PREFIX ) {
-			if (cx->ei->bei_state & CACHE_ENTRY_NO_GRANDKIDS) {
-				bdb_idl_union( cx->ids, cx->tmp );
-			} else {
+			bdb_idl_append( cx->ids, cx->tmp );
+			if ( !(cx->ei->bei_state & CACHE_ENTRY_NO_GRANDKIDS)) {
 				ID *save, idcurs;
 				EntryInfo *ei = cx->ei;
 				int nokids = 1;
 				save = cx->op->o_tmpalloc( BDB_IDL_SIZEOF( cx->tmp ),
 					cx->op->o_tmpmemctx );
 				BDB_IDL_CPY( save, cx->tmp );
-				bdb_idl_union( cx->ids, cx->tmp );
 
 				idcurs = 0;
 				for ( cx->id = bdb_idl_first( save, &idcurs );
@@ -1003,8 +1014,8 @@ hdb_dn2idl(
 	cx.ei = e->e_id ? BEI(e) : &bdb->bi_cache.c_dntree;
 	cx.bdb = bdb;
 	cx.db = cx.bdb->bi_dn2id->bdi_db;
-	cx.prefix = op->ors_scope == LDAP_SCOPE_ONELEVEL
-		? DN_ONE_PREFIX : DN_SUBTREE_PREFIX;
+	cx.prefix = (op->ors_scope == LDAP_SCOPE_ONELEVEL) ?
+		DN_ONE_PREFIX : DN_SUBTREE_PREFIX;
 	cx.ids = ids;
 	cx.buf = stack;
 	cx.op = op;
@@ -1021,6 +1032,10 @@ hdb_dn2idl(
 
 	DBTzero(&cx.data);
 
-	return hdb_dn2idl_internal(&cx);
+	hdb_dn2idl_internal(&cx);
+	if ( !BDB_IDL_IS_ZERO( ids ) && !BDB_IDL_IS_RANGE( ids ))
+		bdb_idl_sort( ids );
+
+	return cx.rc;
 }
 #endif	/* BDB_HIER */
