@@ -1,5 +1,5 @@
 /* filterindex.c - generate the list of candidate entries from a filter */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/filterindex.c,v 1.51.2.3 2005/03/14 22:24:59 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/filterindex.c,v 1.51.2.4 2005/04/29 21:29:06 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 2000-2005 The OpenLDAP Foundation.
@@ -297,12 +297,20 @@ comp_equality_candidates (
         MatchingRule *mr = mra->ma_rule;
         Syntax *sat_syntax;
 	ComponentReference* cr_list, *cr;
+	AttrInfo *ai;
 
         BDB_IDL_ALL( bdb, ids );
 
-	bdb_attr_comp_ref ( op->o_bd->be_private, mra->ma_desc, &cr_list );
-	if( !cr_list || !ca->ca_comp_ref )
+	if ( !ca->ca_comp_ref )
 		return 0;
+
+	ai = bdb_attr_mask( op->o_bd->be_private, mra->ma_desc );
+	if( ai ) {
+		cr_list = ai->ai_cr;
+	}
+	else {
+		return 0;
+	}
 	/* find a component reference to be indexed */
 	sat_syntax = ca->ca_ma_rule->smr_syntax;
 	for ( cr = cr_list ; cr ; cr = cr->cr_next ) {
