@@ -1,8 +1,8 @@
 /* compare.c - DNS SRV backend compare function */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-dnssrv/compare.c,v 1.12.2.1 2003/12/01 00:58:39 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-dnssrv/compare.c,v 1.16.2.2 2005/03/14 22:25:00 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2000-2003 The OpenLDAP Foundation.
+ * Copyright 2000-2005 The OpenLDAP Foundation.
  * Portions Copyright 2000-2003 Kurt D. Zeilenga.
  * All rights reserved.
  *
@@ -27,7 +27,7 @@
 #include <ac/socket.h>
 
 #include "slap.h"
-#include "back-dnssrv.h"
+#include "proto-dnssrv.h"
 
 int
 dnssrv_back_compare(
@@ -39,9 +39,8 @@ dnssrv_back_compare(
 	assert( get_manageDSAit( op ) );
 #endif
 	send_ldap_error( op, rs, LDAP_OTHER,
-			"Operation not supported within naming context" );
+		"Operation not supported within naming context" );
 
 	/* not implemented */
-
 	return 1;
 }
