@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-meta/dncache.c,v 1.10.2.1 2005/01/20 18:04:00 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-meta/dncache.c,v 1.10.2.2 2005/04/29 21:29:08 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1999-2005 The OpenLDAP Foundation.
@@ -23,6 +23,7 @@
 #include "portable.h"
 
 #include <stdio.h>
+#include <ac/string.h>
 
 #include "slap.h"
 #include "../back-ldap/back-ldap.h"
@@ -32,12 +33,12 @@
  * The dncache, at present, maps an entry to the target that holds it.
  */
 
-struct metadncacheentry {
+typedef struct metadncacheentry_t {
 	struct berval	dn;
 	int 		target;
 
 	time_t 		lastupdated;
-};
+} metadncacheentry_t;
 
 /*
  * meta_dncache_cmp
@@ -48,12 +49,11 @@ struct metadncacheentry {
  */
 int
 meta_dncache_cmp(
-		const void *c1,
-		const void *c2
-)
+	const void	*c1,
+	const void	*c2 )
 {
-	struct metadncacheentry *cc1 = ( struct metadncacheentry * )c1;
-	struct metadncacheentry *cc2 = ( struct metadncacheentry * )c2;
+	metadncacheentry_t	*cc1 = ( metadncacheentry_t * )c1;
+	metadncacheentry_t	*cc2 = ( metadncacheentry_t * )c2;
 
 	/*
 	 * case sensitive, because the dn MUST be normalized
@@ -69,12 +69,11 @@ meta_dncache_cmp(
  */
 int
 meta_dncache_dup(
-		void *c1,
-		void *c2
-)
+	void		*c1,
+	void		*c2 )
 {
-	struct metadncacheentry *cc1 = ( struct metadncacheentry * )c1;
-	struct metadncacheentry *cc2 = ( struct metadncacheentry * )c2;
+	metadncacheentry_t	*cc1 = ( metadncacheentry_t * )c1;
+	metadncacheentry_t	*cc2 = ( metadncacheentry_t * )c2;
 	
 	/*
 	 * case sensitive, because the dn MUST be normalized
@@ -90,20 +89,20 @@ meta_dncache_dup(
  */
 int
 meta_dncache_get_target(
-		struct metadncache	*cache,
-		struct berval		*ndn
-)
+	metadncache_t	*cache,
+	struct berval	*ndn )
 {
-	struct metadncacheentry tmp_entry, *entry;
-	time_t curr_time;
-	int target = META_TARGET_NONE;
+	metadncacheentry_t	tmp_entry,
+				*entry;
+	time_t			curr_time;
+	int			target = META_TARGET_NONE;
 
 	assert( cache );
 	assert( ndn );
 
 	tmp_entry.dn = *ndn;
 	ldap_pvt_thread_mutex_lock( &cache->mutex );
-	entry = ( struct metadncacheentry * )avl_find( cache->tree,
+	entry = ( metadncacheentry_t * )avl_find( cache->tree,
 			( caddr_t )&tmp_entry, meta_dncache_cmp );
 
 	if ( entry != NULL ) {
@@ -141,14 +140,14 @@ meta_dncache_get_target(
  */
 int
 meta_dncache_update_entry(
-		struct metadncache      *cache,
-		struct berval		*ndn,
-		int 			target
-)
+	metadncache_t	*cache,
+	struct berval	*ndn,
+	int 		target )
 {
-	struct metadncacheentry *entry, tmp_entry;
-	time_t curr_time = 0L;
-	int err = 0;
+	metadncacheentry_t	*entry,
+				tmp_entry;
+	time_t			curr_time = 0L;
+	int			err = 0;
 
 	assert( cache );
 	assert( ndn );
@@ -169,24 +168,22 @@ meta_dncache_update_entry(
 	tmp_entry.dn = *ndn;
 
 	ldap_pvt_thread_mutex_lock( &cache->mutex );
-	entry = ( struct metadncacheentry * )avl_find( cache->tree,
+	entry = ( metadncacheentry_t * )avl_find( cache->tree,
 			( caddr_t )&tmp_entry, meta_dncache_cmp );
 
 	if ( entry != NULL ) {
 		entry->target = target;
 		entry->lastupdated = curr_time;
+
 	} else {
-		entry = ch_calloc( sizeof( struct metadncacheentry ), 1 );
+		entry = ch_malloc( sizeof( metadncacheentry_t ) + ndn->bv_len + 1 );
 		if ( entry == NULL ) {
 			ldap_pvt_thread_mutex_unlock( &cache->mutex );
 			return -1;
 		}
 
-		ber_dupbv( &entry->dn, ndn );
-		if ( entry->dn.bv_val == NULL ) {
-			ldap_pvt_thread_mutex_unlock( &cache->mutex );
-			return -1;
-		}
+		entry->dn.bv_val = (char *)&entry[ 1 ];
+		AC_MEMCPY( entry->dn.bv_val, ndn->bv_val, ndn->bv_len + 1 );
 		entry->target = target;
 		entry->lastupdated = curr_time;
 
@@ -206,11 +203,11 @@ meta_dncache_update_entry(
  */
 int
 meta_dncache_delete_entry(
-		struct metadncache      *cache,
-		struct berval		*ndn
-)
+	metadncache_t	*cache,
+	struct berval	*ndn )
 {
-	struct metadncacheentry *entry, tmp_entry;
+	metadncacheentry_t	*entry,
+				tmp_entry;
 
 	assert( cache );
 	assert( ndn );
@@ -237,11 +234,8 @@ meta_dncache_delete_entry(
  */
 void
 meta_dncache_free(
-		void *e
-)
+	void		*e )
 {
-	struct metadncacheentry *entry = ( struct metadncacheentry * )e;
-
-	free( entry->dn.bv_val );
+	free( e );
 }
 
