@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/entry-id.c,v 1.46.2.1 2005/01/20 18:04:02 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/entry-id.c,v 1.46.2.2 2005/04/29 21:29:09 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1999-2005 The OpenLDAP Foundation.
@@ -299,11 +299,9 @@ backsql_dn2id(
 			 */
 			rs->sr_matched = NULL;
 			while ( !be_issuffix( op->o_bd, &pdn ) ) {
-				struct berval	dn;
 				char		*matchedDN = NULL;
 	
-				dn = pdn;
-				dnParent( &dn, &pdn );
+				dnParent( &pdn, &pdn );
 	
 				/*
 				 * Empty DN ("") defaults to LDAP_SUCCESS
