@@ -1,8 +1,8 @@
 /* filter.c - routines for parsing and dealing with filters */
-/* $OpenLDAP: pkg/ldap/servers/slapd/filter.c,v 1.115.2.2 2003/12/01 00:58:34 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/filter.c,v 1.125.2.1 2005/01/20 18:03:55 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2003 The OpenLDAP Foundation.
+ * Copyright 1998-2005 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -73,11 +73,7 @@ get_filter(
 	int		err;
 	Filter		f;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( FILTER, ENTRY, "get_filter: conn %d\n", op->o_connid, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_FILTER, "begin get_filter\n", 0, 0, 0 );
-#endif
 	/*
 	 * A filter looks like this coming in:
 	 *	Filter ::= CHOICE {
@@ -125,12 +121,7 @@ get_filter(
 
 	switch ( f.f_choice ) {
 	case LDAP_FILTER_EQUALITY:
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, DETAIL2, 
-			"get_filter: conn %d  EQUALITY\n", op->o_connid, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_FILTER, "EQUALITY\n", 0, 0, 0 );
-#endif
 		err = get_ava( op, ber, &f.f_ava, SLAP_MR_EQUALITY, text );
 		if ( err != LDAP_SUCCESS ) {
 			break;
@@ -140,12 +131,7 @@ get_filter(
 		break;
 
 	case LDAP_FILTER_SUBSTRINGS:
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, DETAIL1, 
-			"get_filter: conn %d  SUBSTRINGS\n", op->o_connid, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_FILTER, "SUBSTRINGS\n", 0, 0, 0 );
-#endif
 		err = get_ssa( op, ber, &f.f_sub, text );
 		if( err != LDAP_SUCCESS ) {
 			break;
@@ -154,12 +140,7 @@ get_filter(
 		break;
 
 	case LDAP_FILTER_GE:
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, DETAIL1, 
-			"get_filter: conn %d  GE\n", op->o_connid, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_FILTER, "GE\n", 0, 0, 0 );
-#endif
 		err = get_ava( op, ber, &f.f_ava, SLAP_MR_ORDERING, text );
 		if ( err != LDAP_SUCCESS ) {
 			break;
@@ -168,12 +149,7 @@ get_filter(
 		break;
 
 	case LDAP_FILTER_LE:
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, DETAIL1, 
-			"get_filter: conn %d  LE\n", op->o_connid, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_FILTER, "LE\n", 0, 0, 0 );
-#endif
 		err = get_ava( op, ber, &f.f_ava, SLAP_MR_ORDERING, text );
 		if ( err != LDAP_SUCCESS ) {
 			break;
@@ -184,12 +160,7 @@ get_filter(
 	case LDAP_FILTER_PRESENT: {
 		struct berval type;
 
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, DETAIL1, 
-			"get_filter: conn %d PRESENT\n", op->o_connid, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_FILTER, "PRESENT\n", 0, 0, 0 );
-#endif
 		if ( ber_scanf( ber, "m", &type ) == LBER_ERROR ) {
 			err = SLAPD_DISCONNECT;
 			*text = "error decoding filter";
@@ -201,6 +172,11 @@ get_filter(
 
 		if( err != LDAP_SUCCESS ) {
 			/* unrecognized attribute description or other error */
+			Debug( LDAP_DEBUG_ANY, 
+				"get_filter: conn %d unknown attribute "
+				"type=%s (%d)\n",
+				op->o_connid, type.bv_val, err );
+
 			f.f_choice = SLAPD_FILTER_COMPUTED;
 			f.f_result = LDAP_COMPARE_FALSE;
 			err = LDAP_SUCCESS;
@@ -212,12 +188,7 @@ get_filter(
 		} break;
 
 	case LDAP_FILTER_APPROX:
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, DETAIL1, 
-			"get_filter: conn %d  APPROX\n", op->o_connid, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_FILTER, "APPROX\n", 0, 0, 0 );
-#endif
 		err = get_ava( op, ber, &f.f_ava, SLAP_MR_EQUALITY_APPROX, text );
 		if ( err != LDAP_SUCCESS ) {
 			break;
@@ -226,12 +197,7 @@ get_filter(
 		break;
 
 	case LDAP_FILTER_AND:
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, DETAIL1, 
-			"get_filter: conn %d  AND\n", op->o_connid, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_FILTER, "AND\n", 0, 0, 0 );
-#endif
 		err = get_filter_list( op, ber, &f.f_and, text );
 		if ( err != LDAP_SUCCESS ) {
 			break;
@@ -244,12 +210,7 @@ get_filter(
 		break;
 
 	case LDAP_FILTER_OR:
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, DETAIL1, 
-			"get_filter: conn %d  OR\n", op->o_connid, 0, 0  );
-#else
 		Debug( LDAP_DEBUG_FILTER, "OR\n", 0, 0, 0 );
-#endif
 		err = get_filter_list( op, ber, &f.f_or, text );
 		if ( err != LDAP_SUCCESS ) {
 			break;
@@ -262,12 +223,7 @@ get_filter(
 		break;
 
 	case LDAP_FILTER_NOT:
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, DETAIL1, 
-			"get_filter: conn %d  NOT\n", op->o_connid, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_FILTER, "NOT\n", 0, 0, 0 );
-#endif
 		(void) ber_skip_tag( ber, &len );
 		err = get_filter( op, ber, &f.f_not, text );
 		if ( err != LDAP_SUCCESS ) {
@@ -295,12 +251,7 @@ get_filter(
 		break;
 
 	case LDAP_FILTER_EXT:
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, DETAIL1, 
-			"get_filter: conn %d  EXTENSIBLE\n", op->o_connid, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_FILTER, "EXTENSIBLE\n", 0, 0, 0 );
-#endif
 
 		err = get_mra( op, ber, &f.f_mra, text );
 		if ( err != LDAP_SUCCESS ) {
@@ -312,14 +263,8 @@ get_filter(
 
 	default:
 		(void) ber_scanf( ber, "x" ); /* skip the element */
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, ERR, 
-			"get_filter: conn %d unknown filter type=%lu\n",
-			op->o_connid, f.f_choice, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "get_filter: unknown filter type=%lu\n",
 			f.f_choice, 0, 0 );
-#endif
 		f.f_choice = SLAPD_FILTER_COMPUTED;
 		f.f_result = SLAPD_COMPARE_UNDEFINED;
 		break;
@@ -338,12 +283,7 @@ get_filter(
 		**filt = f;
 	}
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( FILTER, DETAIL2, 
-		"get_filter: conn %d exit\n", op->o_connid, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_FILTER, "end get_filter %d\n", err, 0, 0 );
-#endif
 
 	return( err );
 }
@@ -359,12 +299,7 @@ get_filter_list( Operation *op, BerElement *ber,
 	ber_len_t	len;
 	char		*last;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( FILTER, ENTRY, 
-		"get_filter_list: conn %d start\n", op->o_connid, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_FILTER, "begin get_filter_list\n", 0, 0, 0 );
-#endif
 	new = f;
 	for ( tag = ber_first_element( ber, &len, &last );
 		tag != LBER_DEFAULT;
@@ -377,12 +312,7 @@ get_filter_list( Operation *op, BerElement *ber,
 	}
 	*new = NULL;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( FILTER, ENTRY, 
-		"get_filter_list: conn %d exit\n", op->o_connid, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_FILTER, "end get_filter_list\n", 0, 0, 0 );
-#endif
 	return( LDAP_SUCCESS );
 }
 
@@ -401,13 +331,9 @@ get_ssa(
 	SubstringsAssertion ssa;
 
 	*text = "error decoding filter";
+	*out = NULL;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( FILTER, ENTRY, 
-		"get_ssa: conn %d  begin\n", op->o_connid, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_FILTER, "begin get_ssa\n", 0, 0, 0 );
-#endif
 	if ( ber_scanf( ber, "{m" /*}*/, &desc ) == LBER_ERROR ) {
 		return SLAPD_DISCONNECT;
 	}
@@ -422,6 +348,10 @@ get_ssa(
 	rc = slap_bv2ad( &desc, &ssa.sa_desc, text );
 
 	if( rc != LDAP_SUCCESS ) {
+		Debug( LDAP_DEBUG_ANY, 
+			"get_ssa: conn %d unknown attribute type=%s (%d)\n",
+			op->o_connid, desc.bv_val, rc );
+
 		/* skip over the rest of this filter */
 		for ( tag = ber_first_element( ber, &len, &last );
 			tag != LBER_DEFAULT;
@@ -466,15 +396,9 @@ get_ssa(
 		default:
 			rc = LDAP_PROTOCOL_ERROR;
 
-#ifdef NEW_LOGGING
-			LDAP_LOG( FILTER, ERR,
-				"get_filter_substring: conn %d  unknown substring choice=%ld\n",
-				op->o_connid, (long)tag, 0 );
-#else
 			Debug( LDAP_DEBUG_FILTER,
 				"  unknown substring choice=%ld\n",
 				(long) tag, 0, 0 );
-#endif
 
 			goto return_error;
 		}
@@ -492,19 +416,13 @@ get_ssa(
 
 		switch ( tag ) {
 		case LDAP_SUBSTRING_INITIAL:
-#ifdef NEW_LOGGING
-			LDAP_LOG( FILTER, DETAIL1,
-				"get_ssa: conn %d  INITIAL\n",
-				op->o_connid, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_FILTER, "  INITIAL\n", 0, 0, 0 );
-#endif
 
 			if ( ssa.sa_initial.bv_val != NULL
 				|| ssa.sa_any != NULL 
 				|| ssa.sa_final.bv_val != NULL )
 			{
-				sl_free( nvalue.bv_val, op->o_tmpmemctx );
+				slap_sl_free( nvalue.bv_val, op->o_tmpmemctx );
 				goto return_error;
 			}
 
@@ -512,16 +430,10 @@ get_ssa(
 			break;
 
 		case LDAP_SUBSTRING_ANY:
-#ifdef NEW_LOGGING
-			LDAP_LOG( FILTER, DETAIL1,
-				"get_ssa: conn %d  ANY\n",
-				op->o_connid, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_FILTER, "  ANY\n", 0, 0, 0 );
-#endif
 
 			if ( ssa.sa_final.bv_val != NULL ) {
-				sl_free( nvalue.bv_val, op->o_tmpmemctx );
+				slap_sl_free( nvalue.bv_val, op->o_tmpmemctx );
 				goto return_error;
 			}
 
@@ -529,16 +441,10 @@ get_ssa(
 			break;
 
 		case LDAP_SUBSTRING_FINAL:
-#ifdef NEW_LOGGING
-			LDAP_LOG( FILTER, DETAIL1, 
-				"get_ssa: conn %d  FINAL\n",
-				op->o_connid, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_FILTER, "  FINAL\n", 0, 0, 0 );
-#endif
 
 			if ( ssa.sa_final.bv_val != NULL ) {
-				sl_free( nvalue.bv_val, op->o_tmpmemctx );
+				slap_sl_free( nvalue.bv_val, op->o_tmpmemctx );
 				goto return_error;
 			}
 
@@ -546,31 +452,19 @@ get_ssa(
 			break;
 
 		default:
-#ifdef NEW_LOGGING
-			LDAP_LOG( FILTER, INFO, 
-				"get_ssa: conn %d  unknown substring type %ld\n",
-				op->o_connid, (long)tag, 0 );
-#else
 			Debug( LDAP_DEBUG_FILTER,
 				"  unknown substring type=%ld\n",
 				(long) tag, 0, 0 );
-#endif
 
 			assert( 0 );
-			sl_free( nvalue.bv_val, op->o_tmpmemctx );
+			slap_sl_free( nvalue.bv_val, op->o_tmpmemctx );
 
 return_error:
-#ifdef NEW_LOGGING
-			LDAP_LOG( FILTER, INFO, 
-				"get_ssa: conn %d  error %ld\n",
-				op->o_connid, (long)rc, 0 );
-#else
 			Debug( LDAP_DEBUG_FILTER, "  error=%ld\n",
 				(long) rc, 0, 0 );
-#endif
-			sl_free( ssa.sa_initial.bv_val, op->o_tmpmemctx );
+			slap_sl_free( ssa.sa_initial.bv_val, op->o_tmpmemctx );
 			ber_bvarray_free_x( ssa.sa_any, op->o_tmpmemctx );
-			sl_free( ssa.sa_final.bv_val, op->o_tmpmemctx );
+			slap_sl_free( ssa.sa_final.bv_val, op->o_tmpmemctx );
 			return rc;
 		}
 
@@ -582,14 +476,9 @@ return_error:
 		**out = ssa;
 	}
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( FILTER, ENTRY, 
-		"get_ssa: conn %d exit\n", op->o_connid, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_FILTER, "end get_ssa\n", 0, 0, 0 );
-#endif
 
-	return LDAP_SUCCESS;
+	return rc /* LDAP_SUCCESS */ ;
 }
 
 void
@@ -640,13 +529,8 @@ filter_free_x( Operation *op, Filter *f )
 		break;
 
 	default:
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, ERR, 
-			"filter_free: unknown filter type %lu\n", f->f_choice, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "filter_free: unknown filter type=%lu\n",
 			f->f_choice, 0, 0 );
-#endif
 		break;
 	}
 
@@ -657,22 +541,31 @@ void
 filter_free( Filter *f )
 {
 	Operation op;
+	Opheader ohdr;
 
-	op.o_tmpmemctx = sl_context( f );
-	op.o_tmpmfuncs = &sl_mfuncs;
+	op.o_hdr = &ohdr;
+	op.o_tmpmemctx = slap_sl_context( f );
+	op.o_tmpmfuncs = &slap_sl_mfuncs;
 	filter_free_x( &op, f );
 }
 
 void
 filter2bv_x( Operation *op, Filter *f, struct berval *fstr )
 {
-	int	i;
-	Filter	*p;
-	struct berval tmp;
-	ber_len_t len;
+	int		i;
+	Filter		*p;
+	struct berval	tmp;
+	static struct berval
+			ber_bvfalse = BER_BVC( "(?=false)" ),
+			ber_bvtrue = BER_BVC( "(?=true)" ),
+			ber_bvundefined = BER_BVC( "(?=undefined)" ),
+			ber_bverror = BER_BVC( "(?=error)" ),
+			ber_bvunknown = BER_BVC( "(?=unknown)" ),
+			ber_bvnone = BER_BVC( "(?=none)" );
+	ber_len_t	len;
 
 	if ( f == NULL ) {
-		ber_str2bv_x( "No filter!", sizeof("No filter!")-1, 1, fstr, op->o_tmpmemctx );
+		ber_dupbv_x( fstr, &ber_bvnone, op->o_tmpmemctx );
 		return;
 	}
 
@@ -746,7 +639,8 @@ filter2bv_x( Operation *op, Filter *f, struct berval *fstr )
 			filter_escape_value_x( &f->f_sub_initial, &tmp, op->o_tmpmemctx );
 
 			fstr->bv_len += tmp.bv_len;
-			fstr->bv_val = op->o_tmprealloc( fstr->bv_val, fstr->bv_len + 1, op->o_tmpmemctx );
+			fstr->bv_val = op->o_tmprealloc( fstr->bv_val, fstr->bv_len + 1,
+				op->o_tmpmemctx );
 
 			snprintf( &fstr->bv_val[len-2], tmp.bv_len+3,
 				/* "(attr=" */ "%s*)",
@@ -758,10 +652,12 @@ filter2bv_x( Operation *op, Filter *f, struct berval *fstr )
 		if ( f->f_sub_any != NULL ) {
 			for ( i = 0; f->f_sub_any[i].bv_val != NULL; i++ ) {
 				len = fstr->bv_len;
-				filter_escape_value_x( &f->f_sub_any[i], &tmp, op->o_tmpmemctx );
+				filter_escape_value_x( &f->f_sub_any[i],
+					&tmp, op->o_tmpmemctx );
 
 				fstr->bv_len += tmp.bv_len + 1;
-				fstr->bv_val = op->o_tmprealloc( fstr->bv_val, fstr->bv_len + 1, op->o_tmpmemctx );
+				fstr->bv_val = op->o_tmprealloc( fstr->bv_val, fstr->bv_len + 1,
+					op->o_tmpmemctx );
 
 				snprintf( &fstr->bv_val[len-1], tmp.bv_len+3,
 					/* "(attr=[init]*[any*]" */ "%s*)",
@@ -776,7 +672,8 @@ filter2bv_x( Operation *op, Filter *f, struct berval *fstr )
 			filter_escape_value_x( &f->f_sub_final, &tmp, op->o_tmpmemctx );
 
 			fstr->bv_len += tmp.bv_len;
-			fstr->bv_val = op->o_tmprealloc( fstr->bv_val, fstr->bv_len + 1, op->o_tmpmemctx );
+			fstr->bv_val = op->o_tmprealloc( fstr->bv_val, fstr->bv_len + 1,
+				op->o_tmpmemctx );
 
 			snprintf( &fstr->bv_val[len-1], tmp.bv_len+3,
 				/* "(attr=[init*][any*]" */ "%s)",
@@ -812,7 +709,8 @@ filter2bv_x( Operation *op, Filter *f, struct berval *fstr )
 			filter2bv_x( op, p, &tmp );
 			
 			fstr->bv_len += tmp.bv_len;
-			fstr->bv_val = op->o_tmprealloc( fstr->bv_val, fstr->bv_len + 1, op->o_tmpmemctx );
+			fstr->bv_val = op->o_tmprealloc( fstr->bv_val, fstr->bv_len + 1,
+				op->o_tmpmemctx );
 
 			snprintf( &fstr->bv_val[len-1], tmp.bv_len + 2, 
 				/*"("*/ "%s)", tmp.bv_val );
@@ -823,47 +721,55 @@ filter2bv_x( Operation *op, Filter *f, struct berval *fstr )
 		break;
 
 	case LDAP_FILTER_EXT: {
-		struct berval ad;
-		filter_escape_value_x( &f->f_mr_value, &tmp, op->o_tmpmemctx );
+			struct berval ad;
+			filter_escape_value_x( &f->f_mr_value, &tmp, op->o_tmpmemctx );
 
-		if ( f->f_mr_desc ) {
-			ad = f->f_mr_desc->ad_cname;
-		} else {
-			ad.bv_len = 0;
-			ad.bv_val = "";
-		}
+			if ( f->f_mr_desc ) {
+				ad = f->f_mr_desc->ad_cname;
+			} else {
+				ad.bv_len = 0;
+				ad.bv_val = "";
+			}
 			
-		fstr->bv_len = ad.bv_len +
-			( f->f_mr_dnattrs ? sizeof(":dn")-1 : 0 ) +
-			( f->f_mr_rule_text.bv_len ? f->f_mr_rule_text.bv_len+1 : 0 ) +
-			tmp.bv_len + ( sizeof("(:=)") - 1 );
-		fstr->bv_val = op->o_tmpalloc( fstr->bv_len + 1, op->o_tmpmemctx );
+			fstr->bv_len = ad.bv_len +
+				( f->f_mr_dnattrs ? sizeof(":dn")-1 : 0 ) +
+				( f->f_mr_rule_text.bv_len ? f->f_mr_rule_text.bv_len+1 : 0 ) +
+				tmp.bv_len + ( sizeof("(:=)") - 1 );
+			fstr->bv_val = op->o_tmpalloc( fstr->bv_len + 1, op->o_tmpmemctx );
 
-		snprintf( fstr->bv_val, fstr->bv_len + 1, "(%s%s%s%s:=%s)",
-			ad.bv_val,
-			f->f_mr_dnattrs ? ":dn" : "",
-			f->f_mr_rule_text.bv_len ? ":" : "",
-			f->f_mr_rule_text.bv_len ? f->f_mr_rule_text.bv_val : "",
-			tmp.bv_val );
-		ber_memfree_x( tmp.bv_val, op->o_tmpmemctx );
+			snprintf( fstr->bv_val, fstr->bv_len + 1, "(%s%s%s%s:=%s)",
+				ad.bv_val,
+				f->f_mr_dnattrs ? ":dn" : "",
+				f->f_mr_rule_text.bv_len ? ":" : "",
+				f->f_mr_rule_text.bv_len ? f->f_mr_rule_text.bv_val : "",
+				tmp.bv_val );
+			ber_memfree_x( tmp.bv_val, op->o_tmpmemctx );
 		} break;
 
 	case SLAPD_FILTER_COMPUTED:
-		ber_str2bv_x(
-			f->f_result == LDAP_COMPARE_FALSE ? "(?=false)" :
-			f->f_result == LDAP_COMPARE_TRUE ? "(?=true)" :
-			f->f_result == SLAPD_COMPARE_UNDEFINED ? "(?=undefined)" :
-			"(?=error)",
-			f->f_result == LDAP_COMPARE_FALSE ? sizeof("(?=false)")-1 :
-			f->f_result == LDAP_COMPARE_TRUE ? sizeof("(?=true)")-1 :
-			f->f_result == SLAPD_COMPARE_UNDEFINED ? sizeof("(?=undefined)")-1 :
-			sizeof("(?=error)")-1,
-			1, fstr, op->o_tmpmemctx );
-		break;
+		switch ( f->f_result ) {
+		case LDAP_COMPARE_FALSE:
+			tmp = ber_bvfalse;
+			break;
 
+		case LDAP_COMPARE_TRUE:
+			tmp = ber_bvtrue;
+			break;
+			
+		case SLAPD_COMPARE_UNDEFINED:
+			tmp = ber_bvundefined;
+			break;
+			
+		default:
+			tmp = ber_bverror;
+			break;
+		}
+
+		ber_dupbv_x( fstr, &tmp, op->o_tmpmemctx );
+		break;
+		
 	default:
-		ber_str2bv_x( "(?=unknown)", sizeof("(?=unknown)")-1,
-			1, fstr, op->o_tmpmemctx );
+		ber_dupbv_x( fstr, &ber_bvunknown, op->o_tmpmemctx );
 		break;
 	}
 }
@@ -872,6 +778,9 @@ void
 filter2bv( Filter *f, struct berval *fstr )
 {
 	Operation op;
+	Opheader ohdr;
+
+	op.o_hdr = &ohdr;
 	op.o_tmpmemctx = NULL;
 	op.o_tmpmfuncs = &ch_mfuncs;
 
@@ -889,7 +798,7 @@ filter_escape_value_x(
 	assert( out );
 
 	i = in->bv_len * 3 + 1;
-	out->bv_val = ctx ? sl_malloc( i, ctx ) : ch_malloc( i );
+	out->bv_val = ctx ? slap_sl_malloc( i, ctx ) : ch_malloc( i );
 	out->bv_len = 0;
 
 	for( i=0; i < in->bv_len ; i++ ) {
@@ -926,12 +835,7 @@ get_simple_vrFilter(
 	int		err;
 	ValuesReturnFilter vrf;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( FILTER, ENTRY, 
-		"get_simple_vrFilter: conn %d\n", op->o_connid, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_FILTER, "begin get_simple_vrFilter\n", 0, 0, 0 );
-#endif
 
 	tag = ber_peek_tag( ber, &len );
 
@@ -947,12 +851,7 @@ get_simple_vrFilter(
 
 	switch ( vrf.vrf_choice ) {
 	case LDAP_FILTER_EQUALITY:
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, DETAIL2, 
-			"get_simple_vrFilter: conn %d  EQUALITY\n", op->o_connid, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_FILTER, "EQUALITY\n", 0, 0, 0 );
-#endif
 		err = get_ava( op, ber, &vrf.vrf_ava, SLAP_MR_EQUALITY, text );
 		if ( err != LDAP_SUCCESS ) {
 			break;
@@ -962,22 +861,12 @@ get_simple_vrFilter(
 		break;
 
 	case LDAP_FILTER_SUBSTRINGS:
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, DETAIL1, 
-			"get_simple_vrFilter: conn %d  SUBSTRINGS\n", op->o_connid, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_FILTER, "SUBSTRINGS\n", 0, 0, 0 );
-#endif
 		err = get_ssa( op, ber, &vrf.vrf_sub, text );
 		break;
 
 	case LDAP_FILTER_GE:
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, DETAIL1, 
-			"get_simple_vrFilter: conn %d  GE\n", op->o_connid, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_FILTER, "GE\n", 0, 0, 0 );
-#endif
 		err = get_ava( op, ber, &vrf.vrf_ava, SLAP_MR_ORDERING, text );
 		if ( err != LDAP_SUCCESS ) {
 			break;
@@ -985,12 +874,7 @@ get_simple_vrFilter(
 		break;
 
 	case LDAP_FILTER_LE:
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, DETAIL1, 
-			"get_simple_vrFilter: conn %d  LE\n", op->o_connid, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_FILTER, "LE\n", 0, 0, 0 );
-#endif
 		err = get_ava( op, ber, &vrf.vrf_ava, SLAP_MR_ORDERING, text );
 		if ( err != LDAP_SUCCESS ) {
 			break;
@@ -1000,12 +884,7 @@ get_simple_vrFilter(
 	case LDAP_FILTER_PRESENT: {
 		struct berval type;
 
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, DETAIL1, 
-			"get_simple_vrFilter: conn %d PRESENT\n", op->o_connid, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_FILTER, "PRESENT\n", 0, 0, 0 );
-#endif
 		if ( ber_scanf( ber, "m", &type ) == LBER_ERROR ) {
 			err = SLAPD_DISCONNECT;
 			*text = "error decoding filter";
@@ -1017,6 +896,11 @@ get_simple_vrFilter(
 
 		if( err != LDAP_SUCCESS ) {
 			/* unrecognized attribute description or other error */
+			Debug( LDAP_DEBUG_ANY, 
+				"get_simple_vrFilter: conn %d unknown "
+				"attribute type=%s (%d)\n",
+				op->o_connid, type.bv_val, err );
+
 			vrf.vrf_choice = SLAPD_FILTER_COMPUTED;
 			vrf.vrf_result = LDAP_COMPARE_FALSE;
 			err = LDAP_SUCCESS;
@@ -1025,12 +909,7 @@ get_simple_vrFilter(
 		} break;
 
 	case LDAP_FILTER_APPROX:
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, DETAIL1, 
-			"get_simple_vrFilter: conn %d  APPROX\n", op->o_connid, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_FILTER, "APPROX\n", 0, 0, 0 );
-#endif
 		err = get_ava( op, ber, &vrf.vrf_ava, SLAP_MR_EQUALITY_APPROX, text );
 		if ( err != LDAP_SUCCESS ) {
 			break;
@@ -1038,12 +917,7 @@ get_simple_vrFilter(
 		break;
 
 	case LDAP_FILTER_EXT:
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, DETAIL1, 
-			"get_simple_vrFilter: conn %d  EXTENSIBLE\n", op->o_connid, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_FILTER, "EXTENSIBLE\n", 0, 0, 0 );
-#endif
 
 		err = get_mra( op, ber, &vrf.vrf_mra, text );
 		if ( err != LDAP_SUCCESS ) {
@@ -1055,14 +929,8 @@ get_simple_vrFilter(
 
 	default:
 		(void) ber_scanf( ber, "x" ); /* skip the element */
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, ERR, 
-			"get_simple_vrFilter: conn %d unknown filter type=%lu\n",
-			op->o_connid, vrf.vrf_choice, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "get_simple_vrFilter: unknown filter type=%lu\n",
 			vrf.vrf_choice, 0, 0 );
-#endif
 		vrf.vrf_choice = SLAPD_FILTER_COMPUTED;
 		vrf.vrf_result = SLAPD_COMPARE_UNDEFINED;
 		break;
@@ -1080,12 +948,7 @@ get_simple_vrFilter(
 		**filt = vrf;
 	}
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( FILTER, DETAIL2, 
-		"get_simple_vrFilter: conn %d exit\n", op->o_connid, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_FILTER, "end get_simple_vrFilter %d\n", err, 0, 0 );
-#endif
 
 	return err;
 }
@@ -1129,12 +992,7 @@ get_vrFilter( Operation *op, BerElement *ber,
 	ber_len_t	len;
 	char		*last;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( FILTER, ENTRY, 
-		"get_vrFilter: conn %d start\n", op->o_connid, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_FILTER, "begin get_vrFilter\n", 0, 0, 0 );
-#endif
 
 	tag = ber_peek_tag( ber, &len );
 
@@ -1161,12 +1019,7 @@ get_vrFilter( Operation *op, BerElement *ber,
 	}
 	*n = NULL;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( FILTER, ENTRY, 
-		"get_vrFilter: conn %d exit\n", op->o_connid, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_FILTER, "end get_vrFilter\n", 0, 0, 0 );
-#endif
 	return( LDAP_SUCCESS );
 }
 
@@ -1212,13 +1065,8 @@ vrFilter_free( Operation *op, ValuesReturnFilter *vrf )
 			break;
 
 		default:
-#ifdef NEW_LOGGING
-			LDAP_LOG( FILTER, ERR, 
-				"filter_free: unknown filter type %lu\n", vrf->vrf_choice, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY, "filter_free: unknown filter type=%lu\n",
 				vrf->vrf_choice, 0, 0 );
-#endif
 			break;
 		}
 
@@ -1250,7 +1098,8 @@ vrFilter2bv( Operation *op, ValuesReturnFilter *vrf, struct berval *fstr )
 		simple_vrFilter2bv( op, p, &tmp );
 			
 		fstr->bv_len += tmp.bv_len;
-		fstr->bv_val = op->o_tmprealloc( fstr->bv_val, fstr->bv_len + 1, op->o_tmpmemctx );
+		fstr->bv_val = op->o_tmprealloc( fstr->bv_val, fstr->bv_len + 1,
+			op->o_tmpmemctx );
 
 		snprintf( &fstr->bv_val[len-1], tmp.bv_len + 2, 
 			/*"("*/ "%s)", tmp.bv_val );
@@ -1266,7 +1115,8 @@ simple_vrFilter2bv( Operation *op, ValuesReturnFilter *vrf, struct berval *fstr
 	ber_len_t len;
 
 	if ( vrf == NULL ) {
-		ber_str2bv_x( "No filter!", sizeof("No filter!")-1, 1, fstr, op->o_tmpmemctx );
+		ber_str2bv_x( "No filter!", sizeof("No filter!")-1, 1, fstr,
+			op->o_tmpmemctx );
 		return;
 	}
 
@@ -1340,7 +1190,8 @@ simple_vrFilter2bv( Operation *op, ValuesReturnFilter *vrf, struct berval *fstr
 			filter_escape_value_x( &vrf->vrf_sub_initial, &tmp, op->o_tmpmemctx );
 
 			fstr->bv_len += tmp.bv_len;
-			fstr->bv_val = op->o_tmprealloc( fstr->bv_val, fstr->bv_len + 1, op->o_tmpmemctx );
+			fstr->bv_val = op->o_tmprealloc( fstr->bv_val, fstr->bv_len + 1,
+				op->o_tmpmemctx );
 
 			snprintf( &fstr->bv_val[len-2], tmp.bv_len+3,
 				/* "(attr=" */ "%s*)",
@@ -1353,10 +1204,12 @@ simple_vrFilter2bv( Operation *op, ValuesReturnFilter *vrf, struct berval *fstr
 			int i;
 			for ( i = 0; vrf->vrf_sub_any[i].bv_val != NULL; i++ ) {
 				len = fstr->bv_len;
-				filter_escape_value_x( &vrf->vrf_sub_any[i], &tmp, op->o_tmpmemctx );
+				filter_escape_value_x( &vrf->vrf_sub_any[i], &tmp,
+					op->o_tmpmemctx );
 
 				fstr->bv_len += tmp.bv_len + 1;
-				fstr->bv_val = op->o_tmprealloc( fstr->bv_val, fstr->bv_len + 1, op->o_tmpmemctx );
+				fstr->bv_val = op->o_tmprealloc( fstr->bv_val,
+					fstr->bv_len + 1, op->o_tmpmemctx );
 
 				snprintf( &fstr->bv_val[len-1], tmp.bv_len+3,
 					/* "(attr=[init]*[any*]" */ "%s*)",
@@ -1371,7 +1224,8 @@ simple_vrFilter2bv( Operation *op, ValuesReturnFilter *vrf, struct berval *fstr
 			filter_escape_value_x( &vrf->vrf_sub_final, &tmp, op->o_tmpmemctx );
 
 			fstr->bv_len += tmp.bv_len;
-			fstr->bv_val = op->o_tmprealloc( fstr->bv_val, fstr->bv_len + 1, op->o_tmpmemctx );
+			fstr->bv_val = op->o_tmprealloc( fstr->bv_val, fstr->bv_len + 1,
+				op->o_tmpmemctx );
 
 			snprintf( &fstr->bv_val[len-1], tmp.bv_len+3,
 				/* "(attr=[init*][any*]" */ "%s)",
@@ -1404,7 +1258,8 @@ simple_vrFilter2bv( Operation *op, ValuesReturnFilter *vrf, struct berval *fstr
 			
 		fstr->bv_len = ad.bv_len +
 			( vrf->vrf_mr_dnattrs ? sizeof(":dn")-1 : 0 ) +
-			( vrf->vrf_mr_rule_text.bv_len ? vrf->vrf_mr_rule_text.bv_len+1 : 0 ) +
+			( vrf->vrf_mr_rule_text.bv_len
+				? vrf->vrf_mr_rule_text.bv_len+1 : 0 ) +
 			tmp.bv_len + ( sizeof("(:=)") - 1 );
 		fstr->bv_val = op->o_tmpalloc( fstr->bv_len + 1, op->o_tmpmemctx );
 
@@ -1422,12 +1277,12 @@ simple_vrFilter2bv( Operation *op, ValuesReturnFilter *vrf, struct berval *fstr
 		ber_str2bv_x(
 			vrf->vrf_result == LDAP_COMPARE_FALSE ? "(?=false)" :
 			vrf->vrf_result == LDAP_COMPARE_TRUE ? "(?=true)" :
-			vrf->vrf_result == SLAPD_COMPARE_UNDEFINED ? "(?=undefined)" :
-			"(?=error)",
+			vrf->vrf_result == SLAPD_COMPARE_UNDEFINED
+				? "(?=undefined)" : "(?=error)",
 			vrf->vrf_result == LDAP_COMPARE_FALSE ? sizeof("(?=false)")-1 :
 			vrf->vrf_result == LDAP_COMPARE_TRUE ? sizeof("(?=true)")-1 :
-			vrf->vrf_result == SLAPD_COMPARE_UNDEFINED ? sizeof("(?=undefined)")-1 :
-			sizeof("(?=error)")-1,
+			vrf->vrf_result == SLAPD_COMPARE_UNDEFINED
+				? sizeof("(?=undefined)")-1 : sizeof("(?=error)")-1,
 			1, fstr, op->o_tmpmemctx );
 		break;
 
