@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/slapi/plugin.c,v 1.15.2.3 2003/12/01 00:58:45 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/slapi/plugin.c,v 1.26.2.2 2005/03/14 22:25:03 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2002-2003 The OpenLDAP Foundation.
+ * Copyright 2002-2005 The OpenLDAP Foundation.
  * Portions Copyright 1997,2002-2003 IBM Corporation.
  * All rights reserved.
  *
@@ -24,13 +24,14 @@
 #include <ldap_pvt_thread.h>
 #include <slap.h>
 #include <slapi.h>
+#include <lutil.h>
 
 /*
  * Note: if ltdl.h is not available, slapi should not be compiled
  */
 #include <ltdl.h>
 
-static int loadPlugin( Slapi_PBlock *, const char *, const char *, int, 
+static int slapi_int_load_plugin( Slapi_PBlock *, const char *, const char *, int, 
 	SLAPI_FUNC *, lt_dlhandle * );
 
 /* pointer to link list of extended objects */
@@ -39,7 +40,7 @@ static ExtendedOp *pGExtendedOps = NULL;
 static Slapi_PBlock *pGPlugins = NULL;
 
 /*********************************************************************
- * Function Name:      newPlugin
+ * Function Name:      plugin_pblock_new
  *
  * Description:        This routine creates a new Slapi_PBlock structure,
  *                     loads in the plugin module and executes the init
@@ -61,11 +62,9 @@ static Slapi_PBlock *pGPlugins = NULL;
  * Messages:           None
  *********************************************************************/
 
-Slapi_PBlock *
-newPlugin(
+static Slapi_PBlock *
+plugin_pblock_new(
 	int type, 
-	const char *path, 
-	const char *initfunc, 
 	int argc, 
 	char *argv[] ) 
 {
@@ -73,6 +72,9 @@ newPlugin(
 	Slapi_PluginDesc *pPluginDesc = NULL;
 	lt_dlhandle	hdLoadHandle;
 	int		rc;
+	char **av2 = NULL, **ppPluginArgv;
+	char *path = argv[2];
+	char *initfunc = argv[3];
 
 	pPlugin = slapi_pblock_new();
 	if ( pPlugin == NULL ) {
@@ -90,19 +92,35 @@ newPlugin(
 		goto done;
 	}
 
-	rc = slapi_pblock_set( pPlugin, SLAPI_PLUGIN_ARGV, (void *)argv );
+	av2 = ldap_charray_dup( argv );
+	if ( !av2 ) {
+		rc = LDAP_NO_MEMORY;
+		goto done;
+	}
+
+	if ( argc > 0 ) {
+		ppPluginArgv = &av2[4];
+	} else {
+		ppPluginArgv = NULL;
+	}
+	rc = slapi_pblock_set( pPlugin, SLAPI_PLUGIN_ARGV, (void *)ppPluginArgv );
 	if ( rc != 0 ) { 
 		goto done;
 	}
 
-	rc = loadPlugin( pPlugin, path, initfunc, TRUE, NULL, &hdLoadHandle );
+	rc = slapi_pblock_set( pPlugin, SLAPI_X_CONFIG_ARGV, (void *)av2 );
+	if ( rc != 0 ) { 
+		goto done;
+	}
+
+	rc = slapi_int_load_plugin( pPlugin, path, initfunc, TRUE, NULL, &hdLoadHandle );
 	if ( rc != 0 ) {
 		goto done;
 	}
 
 	if ( slapi_pblock_get( pPlugin, SLAPI_PLUGIN_DESCRIPTION, (void **)&pPluginDesc ) == 0 &&
 	     pPluginDesc != NULL ) {
-		slapi_log_error(SLAPI_LOG_TRACE, "newPlugin",
+		slapi_log_error(SLAPI_LOG_TRACE, "plugin_pblock_new",
 				"Registered plugin %s %s [%s] (%s)\n",
 				pPluginDesc->spd_id,
 				pPluginDesc->spd_version,
@@ -114,13 +132,16 @@ done:
 	if ( rc != 0 && pPlugin != NULL ) {
 		slapi_pblock_destroy( pPlugin );
 		pPlugin = NULL;
+		if ( av2 ) {
+			ldap_charray_free( av2 );
+		}
 	}
 
 	return pPlugin;
 } 
 
 /*********************************************************************
- * Function Name:      insertPlugin
+ * Function Name:      slapi_int_register_plugin
  *
  * Description:        insert the slapi_pblock structure to the end of the plugin
  *                     list 
@@ -136,7 +157,7 @@ done:
  * Messages:           None
  *********************************************************************/
 int 
-insertPlugin(
+slapi_int_register_plugin(
 	Backend *be, 
 	Slapi_PBlock *pPB )
 { 
@@ -174,7 +195,7 @@ insertPlugin(
 }
        
 /*********************************************************************
- * Function Name:      getAllPluginFuncs
+ * Function Name:      slapi_int_get_plugins
  *
  * Description:        get the desired type of function pointers defined 
  *                     in all the plugins 
@@ -190,7 +211,7 @@ insertPlugin(
  * Messages:           None
  *********************************************************************/
 int 
-getAllPluginFuncs(
+slapi_int_get_plugins(
 	Backend *be, 		
 	int functype, 
 	SLAPI_FUNC **ppFuncPtrs )
@@ -331,7 +352,7 @@ createExtendedOp()
 
 
 /*********************************************************************
- * Function Name:      removeExtendedOp
+ * Function Name:      slapi_int_unregister_extop
  *
  * Description:        This routine removes the ExtendedOp structures 
  *					   asscoiated with a particular extended operation 
@@ -349,7 +370,7 @@ createExtendedOp()
  * Messages:           None
  *********************************************************************/
 void
-removeExtendedOp(
+slapi_int_unregister_extop(
 	Backend *pBE, 
 	ExtendedOp **opList, 
 	Slapi_PBlock *pPB )
@@ -398,7 +419,7 @@ removeExtendedOp(
 
 
 /*********************************************************************
- * Function Name:      newExtendedOp
+ * Function Name:      slapi_int_register_extop
  *
  * Description:        This routine creates a new ExtendedOp structure, loads
  *                     in the extended op module and put the extended op function address
@@ -417,7 +438,7 @@ removeExtendedOp(
  * Messages:           None
  *********************************************************************/
 int 
-newExtendedOp(
+slapi_int_register_extop(
 	Backend *pBE, 	
 	ExtendedOp **opList, 
 	Slapi_PBlock *pPB )
@@ -449,7 +470,7 @@ newExtendedOp(
 	}
 
 	rc = slapi_pblock_get( pPB,SLAPI_PLUGIN_EXT_OP_OIDLIST, &pTmpOIDs );
-	if ( rc != LDAP_SUCCESS ) {
+	if ( rc != 0 ) {
 		rc = LDAP_OTHER;
 		goto error_return;
 	}
@@ -485,7 +506,7 @@ error_return:
 }
 
 /*********************************************************************
- * Function Name:      getPluginFunc
+ * Function Name:      slapi_int_get_extop_plugin
  *
  * Description:        This routine gets the function address for a given function
  *                     name.
@@ -501,7 +522,7 @@ error_return:
  * Messages:           None
  *********************************************************************/
 int 
-getPluginFunc(
+slapi_int_get_extop_plugin(
 	struct berval *reqoid, 		
 	SLAPI_FUNC *pFuncAddr ) 
 {
@@ -532,12 +553,12 @@ getPluginFunc(
 }
 
 /***************************************************************************
- * This function is similar to getPluginFunc above. except it returns one OID
+ * This function is similar to slapi_int_get_extop_plugin above. except it returns one OID
  * per call. It is called from root_dse_info (root_dse.c).
  * The function is a modified version of get_supported_extop (file extended.c).
  ***************************************************************************/
 struct berval *
-ns_get_supported_extop( int index )
+slapi_int_get_supported_extop( int index )
 {
         ExtendedOp	*ext;
 
@@ -554,7 +575,7 @@ ns_get_supported_extop( int index )
 }
 
 /*********************************************************************
- * Function Name:      loadPlugin
+ * Function Name:      slapi_int_load_plugin
  *
  * Description:        This routine loads the specified DLL, gets and executes the init function
  *                     if requested.
@@ -578,7 +599,7 @@ ns_get_supported_extop( int index )
  *********************************************************************/
 
 static int 
-loadPlugin(
+slapi_int_load_plugin(
 	Slapi_PBlock	*pPlugin,
 	const char	*path,
 	const char	*initfunc, 
@@ -624,7 +645,7 @@ loadPlugin(
  * Special support for computed attribute plugins
  */
 int 
-doPluginFNs(
+slapi_int_call_plugins(
 	Backend		*be, 	
 	int		funcType, 
 	Slapi_PBlock	*pPB )
@@ -637,7 +658,7 @@ doPluginFNs(
 		return 1;
 	}
 
-	rc = getAllPluginFuncs( be, funcType, &tmpPlugin );
+	rc = slapi_int_get_plugins( be, funcType, &tmpPlugin );
 	if ( rc != LDAP_SUCCESS || tmpPlugin == NULL ) {
 		/* Nothing to do, front-end should ignore. */
 		return 1;
@@ -672,7 +693,7 @@ doPluginFNs(
 }
 
 int
-netscape_plugin(
+slapi_int_read_config(
 	Backend		*be, 		
 	const char	*fname, 
 	int		lineno, 
@@ -681,7 +702,6 @@ netscape_plugin(
 {
 	int		iType = -1;
 	int		numPluginArgc = 0;
-	char		**ppPluginArgv = NULL;
 
 	if ( argc < 4 ) {
 		fprintf( stderr,
@@ -707,11 +727,6 @@ netscape_plugin(
 	}
 	
 	numPluginArgc = argc - 4;
-	if ( numPluginArgc > 0 ) {
-		ppPluginArgv = &argv[4];
-	} else {
-		ppPluginArgv = NULL;
-	}
 
 	if ( iType == SLAPI_PLUGIN_PREOPERATION ||
 		  	iType == SLAPI_PLUGIN_EXTENDEDOP ||
@@ -720,24 +735,23 @@ netscape_plugin(
 		int rc;
 		Slapi_PBlock *pPlugin;
 
-		pPlugin = newPlugin( iType, argv[2], argv[3], 
-					numPluginArgc, ppPluginArgv );
+		pPlugin = plugin_pblock_new( iType, numPluginArgc, argv );
 		if (pPlugin == NULL) {
 			return 1;
 		}
 
 		if (iType == SLAPI_PLUGIN_EXTENDEDOP) {
-			rc = newExtendedOp(be, &pGExtendedOps, pPlugin);
+			rc = slapi_int_register_extop(be, &pGExtendedOps, pPlugin);
 			if ( rc != LDAP_SUCCESS ) {
 				slapi_pblock_destroy( pPlugin );
 				return 1;
 			}
 		}
 
-		rc = insertPlugin( be, pPlugin );
+		rc = slapi_int_register_plugin( be, pPlugin );
 		if ( rc != LDAP_SUCCESS ) {
 			if ( iType == SLAPI_PLUGIN_EXTENDEDOP ) {
-				removeExtendedOp( be, &pGExtendedOps, pPlugin );
+				slapi_int_unregister_extop( be, &pGExtendedOps, pPlugin );
 			}
 			slapi_pblock_destroy( pPlugin );
 			return 1;
@@ -747,8 +761,40 @@ netscape_plugin(
 	return 0;
 }
 
+void
+slapi_int_plugin_unparse(
+	Backend *be,
+	BerVarray *out
+)
+{
+	Slapi_PBlock *pp;
+	int i, j, rc;
+	char **argv, ibuf[32], *ptr;
+	struct berval idx, bv;
+
+	*out = NULL;
+	idx.bv_val = ibuf;
+	i = 0;
+	for ( pp=be->be_pb; pp; slapi_pblock_get( pp, SLAPI_IBM_PBLOCK, &pp ) ) {
+		slapi_pblock_get( pp, SLAPI_X_CONFIG_ARGV, &argv );
+		idx.bv_len = sprintf( idx.bv_val, "{%d}", i );
+		bv.bv_len = idx.bv_len;
+		for (j=1; argv[j]; j++) {
+			bv.bv_len += strlen(argv[j]);
+			if ( j ) bv.bv_len++;
+		}
+		bv.bv_val = ch_malloc( bv.bv_len + 1 );
+		ptr = lutil_strcopy( bv.bv_val, ibuf );
+		for (j=1; argv[j]; j++) {
+			if ( j ) *ptr++ = ' ';
+			ptr = lutil_strcopy( ptr, argv[j] );
+		}
+		ber_bvarray_add( out, &bv );
+	}
+}
+
 int
-slapi_init(void)
+slapi_int_initialize(void)
 {
 	if ( ldap_pvt_thread_mutex_init( &slapi_hn_mutex ) ) {
 		return -1;
@@ -767,7 +813,7 @@ slapi_init(void)
 		return -1;
 	}
 
-	if ( slapi_x_init_object_extensions() != 0 ) {
+	if ( slapi_int_init_object_extensions() != 0 ) {
 		return -1;
 	}
 
