@@ -1,5 +1,5 @@
 /* unbind.c - decode an ldap unbind operation and pass it to a backend db */
-/* $OpenLDAP: pkg/ldap/servers/slapd/unbind.c,v 1.23.2.1 2005/01/20 18:03:56 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/unbind.c,v 1.23.2.2 2005/04/29 21:29:06 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2005 The OpenLDAP Foundation.
@@ -47,8 +47,15 @@ do_unbind( Operation *op, SlapReply *rs )
 	Statslog( LDAP_DEBUG_STATS, "%s UNBIND\n", op->o_log_prefix,
 		0, 0, 0, 0 );
 
+	if ( frontendDB->be_unbind ) {
+		op->o_bd = frontendDB;
+		(void)frontendDB->be_unbind( op, rs );
+		op->o_bd = NULL;
+	}
+
 	/* pass the unbind to all backends */
-	backend_unbind( op, rs );
+	(void)backend_unbind( op, rs );
 
 	return 0;
 }
+
